@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright (C) 2004-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -25,7 +25,7 @@
 # - speak.txt
 # - katochan が無い場合の処理.(本体の方のpopup menuなども含めて)
 # - 設定ダイアログ : [会話/反応]タブ -> [SEND SSTP/1.1] or [SHIORI]
-# - 見切れ連続20s、もしくは画面内で静止20sでアニメーション記述ミスと見なし自動的に落ちる
+# - 見切れ連続20[s]、もしくは画面内で静止20[s]でアニメーション記述ミスと見なし自動的に落ちる
 # - 発言中にバルーンをダブルクリックで即閉じ
 # - @ゴースト名は#nameと#forには使えない. もし書いても無視されすべて有効になる
 # - 連続落し不可指定
@@ -48,9 +48,11 @@
 
 import os
 import random
+import logging
 
-import gtk
-import glib
+from gi.repository import Gtk
+from gi.repository import Gdk
+from gi.repository import GLib
 import cairo
 
 import ninix.pix
@@ -59,8 +61,8 @@ import ninix.home
 
 class Menu:
 
-    def __init__(self, callback, accelgroup):
-        self.callback = callback
+    def __init__(self, accelgroup):
+        self.request_parent = lambda *a: None # dummy
         ui_info = '''
         <ui>
           <popup name='popup'>
@@ -74,21 +76,19 @@ class Menu:
         '''
         self.__menu_list = {
             'settings': [('Settings', None, _('Settings...(_O)'), None,
-                          '', self.callback['edit_preferences']),
+                          '', lambda *a: self.request_parent('NOTIFY', 'edit_preferences')),
                          '/ui/popup/Settings'],
-            'katochan': [('Katochan', None,_('Katochan(_K)'), None),
+            'katochan': [('Katochan', None, _('Katochan(_K)'), None),
                          '/ui/popup/Katochan'],
             'exit':     [('Exit', None,_('Exit(_Q)'), None,
-                          '', self.callback['close']),
+                          '', lambda *a: self.request_parent('NOTIFY', 'close')),
                          '/ui/popup/Exit'],
             }
         self.__katochan_list = None
-        actions = gtk.ActionGroup('Actions')
-        entry = []
-        for value in self.__menu_list.itervalues():
-            entry.append(value[0])
+        actions = Gtk.ActionGroup('Actions')
+        entry = [value[0] for value in self.__menu_list.values()]
         actions.add_actions(tuple(entry))
-        ui_manager = gtk.UIManager()
+        ui_manager = Gtk.UIManager()
         ui_manager.insert_action_group(actions, 0)
         ui_manager.add_ui_from_string(ui_info)
         self.__popup_menu = ui_manager.get_widget('/ui/popup')
@@ -96,20 +96,25 @@ class Menu:
             path = self.__menu_list[key][-1]
             self.__menu_list[key][1] = ui_manager.get_widget(path)
 
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
     def popup(self, button):
-        katochan_list = self.callback['get_katochan_list']()
+        katochan_list = self.request_parent('GET', 'get_katochan_list')
         self.__set_katochan_menu(katochan_list)
         self.__popup_menu.popup(
-            None, None, None, button, gtk.get_current_event_time())
+            None, None, None, None, button, Gtk.get_current_event_time())
 
     def __set_katochan_menu(self, list): ## FIXME
         key = 'katochan'
         if list:
-            menu = gtk.Menu()
+            menu = Gtk.Menu()
             for katochan in list:
-                item = gtk.MenuItem(katochan['name'])
+                item = Gtk.MenuItem(katochan['name'])
                 item.connect(
-                    'activate', self.callback['select_katochan'], (katochan))
+                    'activate',
+                    lambda a, k: self.request_parent('NOTIFY', 'select_katochan', k),
+                    (katochan))
                 menu.add(item)
                 item.show()
             self.__menu_list[key][1].set_submenu(menu)
@@ -127,7 +132,7 @@ class Nekoninni:
         self.skin = None
         self.katochan = None
 
-    def notify_observer(self, event, args): ## FIXME
+    def observer_update(self, event, args):
         if event in ['set position', 'set surface']:
             if self.skin is not None:
                 self.skin.set_position()
@@ -142,7 +147,7 @@ class Nekoninni:
         elif event == 'finalize':
             self.finalize()
         else:
-            #print 'OBSERVER(nekodorif): ignore -', event
+            ##logging.debug('OBSERVER(nekodorif): ignore - {0}'.format(event))
             pass
 
     def load(self, dir, katochan, target):
@@ -150,28 +155,33 @@ class Nekoninni:
             return 0
         self.dir = dir
         self.target = target
-        self.target.set_observer(self)
-        self.accelgroup = gtk.AccelGroup()
+        self.target.attach_observer(self)
+        self.accelgroup = Gtk.AccelGroup()
         scale = self.target.get_surface_scale()
-        skin_callback ={
-            'finalize': self.finalize,
-            'has_katochan': self.has_katochan,
-            'drop_katochan': self.drop_katochan,
-            'edit_preferences': self.edit_preferences,
-            'close': self.close,
-            'get_katochan_list': lambda *a: self.katochan_list,
-            'select_katochan': self.select_katochan,
-            }
-        self.skin = Skin(self.dir, self.accelgroup, skin_callback, scale)
+        self.skin = Skin(self.dir, self.accelgroup, scale)
+        self.skin.set_responsible(self.handle_request)
         if self.skin is None:
             return 0
         self.katochan_list = katochan
         self.katochan = None
         self.launch_katochan(self.katochan_list[0])
         self.__running = 1
-        glib.timeout_add(50, self.do_idle_tasks) # 50ms
+        GLib.timeout_add(50, self.do_idle_tasks) # 50[ms]
         return 1
 
+    def handle_request(self, event_type, event, *arglist, **argdict):
+        assert event_type in ['GET', 'NOTIFY']
+        handlers = {
+            'get_katochan_list': lambda *a: self.katochan_list,
+            'get_mode': lambda *a: self.mode,
+            }
+        handler = handlers.get(event,
+                               getattr(self, event,
+                                       lambda *a: None)) ## FIXME
+        result = handler(*arglist, **argdict)
+        if event_type == 'GET':
+            return result
+
     def do_idle_tasks(self):
         if not self.__running:
             return False
@@ -194,14 +204,12 @@ class Nekoninni:
                 self.katochan.get_category(),
                 self.katochan.get_kinoko_flag(),
                 self.katochan.get_target())
-        ##print 'KATOCHAN:', args
-        ##print 'TARGET:', args[4]
-        self.target.notify_event('OnNekodorifObject%s' % event, *args)
+        self.target.notify_event('OnNekodorifObject{0}'.format(event), *args)
 
     def has_katochan(self):
         return int(self.katochan is not None)
 
-    def select_katochan(self, widget, args):
+    def select_katochan(self, args):
         self.launch_katochan(args)
 
     def drop_katochan(self):
@@ -215,21 +223,16 @@ class Nekoninni:
     def launch_katochan(self, katochan):
         if self.katochan:
             self.katochan.destroy()
-        katochan_callback = {
-            'finalize': self.finalize,
-            'send_event': self.send_event,
-            'get_mode': lambda *a: self.mode,
-            'delete_katochan': self.delete_katochan,
-            }
-        self.katochan = Katochan(katochan_callback, self.target)
+        self.katochan = Katochan(self.target)
+        self.katochan.set_responsible(self.handle_request)
         self.katochan.load(katochan)
 
-    def edit_preferences(self, action):
+    def edit_preferences(self):
         pass
 
     def finalize(self):
         self.__running = 0
-        self.target.delete_observer(self)
+        self.target.detach_observer(self)
         if self.katochan is not None:
             self.katochan.destroy()
         if self.skin is not None:
@@ -237,25 +240,23 @@ class Nekoninni:
         ##if self.balloon is not None:
         ##    self.balloon.destroy()
 
-    def close(self, action):
+    def close(self):
         self.finalize()
 
 
 class Skin:
 
-    def __init__(self, dir, accelgroup, callback, scale):
+    def __init__(self, dir, accelgroup, scale):
         self.dir = dir
         self.accelgroup = accelgroup
-        self.callback = callback
+        self.request_parent = lambda *a: None # dummy
+        self.dragged = False
         self.x_root = None
         self.y_root = None
         self.__scale = scale
-        menu_callback = {}
-        for key in ['edit_preferences', 'close',
-                    'get_katochan_list', 'select_katochan']:
-            menu_callback[key] = self.callback[key]
-        self.__menu = Menu(menu_callback, self.accelgroup)
-        path = os.path.join(self.dir, 'omni.txt')
+        self.__menu = Menu(self.accelgroup)
+        self.__menu.set_responsible(self.handle_request)
+        path = os.path.join(self.dir, b'omni.txt')
         self.omni = int(os.path.isfile(path) and os.path.getsize(path) == 0)
         self.window = ninix.pix.TransparentWindow()
         name, top_dir = ninix.home.read_profile_txt(dir) # XXX
@@ -264,17 +265,18 @@ class Skin:
         self.window.set_resizable(False)
         self.window.set_focus_on_map(False)
         self.window.connect('delete_event', self.delete)
+        self.window.connect('key_press_event', self.key_press)
         self.window.realize()
-        self.window.window.set_back_pixmap(None, False)
+        self.window.get_window().set_background_pattern(None) # XXX
         self.window.add_accel_group(self.accelgroup)
-        self.darea = gtk.DrawingArea()
-        self.darea.set_events(gtk.gdk.EXPOSURE_MASK|
-                              gtk.gdk.BUTTON_PRESS_MASK|
-                              gtk.gdk.BUTTON_RELEASE_MASK|
-                              gtk.gdk.POINTER_MOTION_MASK|
-                              gtk.gdk.POINTER_MOTION_HINT_MASK|
-                              gtk.gdk.LEAVE_NOTIFY_MASK)
-        self.darea.connect('expose_event', self.redraw)
+        self.darea = Gtk.DrawingArea()
+        self.darea.set_events(Gdk.EventMask.EXPOSURE_MASK|
+                              Gdk.EventMask.BUTTON_PRESS_MASK|
+                              Gdk.EventMask.BUTTON_RELEASE_MASK|
+                              Gdk.EventMask.POINTER_MOTION_MASK|
+                              Gdk.EventMask.POINTER_MOTION_HINT_MASK|
+                              Gdk.EventMask.LEAVE_NOTIFY_MASK)
+        self.darea.connect('draw', self.redraw)
         self.darea.connect('button_press_event', self.button_press)
         self.darea.connect('button_release_event', self.button_release)
         self.darea.connect('motion_notify_event', self.motion_notify)
@@ -287,76 +289,85 @@ class Skin:
         self.set_position(reset=1)
         self.window.show()
 
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
+    def handle_request(self, event_type, event, *arglist, **argdict):
+        assert event_type in ['GET', 'NOTIFY']
+        handlers = {
+            }
+        handler = handlers.get(event, getattr(self, event, None))
+        if handler is None:
+            result = self.request_parent(
+                event_type, event, *arglist, **argdict)
+        else:
+            result = handler(*arglist, **argdict)
+        if event_type == 'GET':
+            return result
+
     def set_scale(self, scale):
         self.__scale = scale
         self.set_surface()
         self.set_position()
 
-    def redraw(self, darea, event):
-        cr = darea.window.cairo_create()
-        cr.save()
-        cr.set_operator(cairo.OPERATOR_CLEAR)
+    def redraw(self, widget, cr):
+        scale = self.__scale
+        cr.scale(scale / 100.0, scale / 100.0)
+        cr.set_source_surface(self.image_surface, 0, 0)
+        cr.set_operator(cairo.OPERATOR_SOURCE)
         cr.paint()
-        cr.restore()
-        cr.set_source_pixbuf(self.current_surface_pixbuf, 0, 0)
-        ##cr.paint_with_alpha(self.__alpha_channel)
-        cr.paint()
-        del cr
+        self.window.update_shape(cr.get_target())
 
     def delete(self, widget, event):
-        self.callback['finalize']()
+        self.request_parent('NOTIFY', 'finalize')
+
+    def key_press(self, window, event):
+        if event.get_state() & (Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK):
+            if event.keyval == Gdk.KEY_F12:
+                logging.info('reset skin position')
+                self.set_position(reset=1)
+        return True
 
     def destroy(self): ## FIXME
         self.window.destroy()
 
-    def button_press(self,  widget, event): ## FIXME
+    def button_press(self,  widget, event):
         self.x_root = event.x_root
         self.y_root = event.y_root
         if event.button == 1:
-            if event.type == gtk.gdk.BUTTON_PRESS:
-                ##self.window.begin_move_drag(
-                ##    event.button, int(event.x_root), int(event.y_root),
-                ##    gtk.get_current_event_time())
+            if event.type == Gdk.EventType.BUTTON_PRESS:
                 pass
-            elif event.type == gtk.gdk._2BUTTON_PRESS: # double click
-                if self.callback['has_katochan']():
+            elif event.type == Gdk.EventType._2BUTTON_PRESS: # double click
+                if self.request_parent('GET', 'has_katochan'):
                     self.start() ## FIXME
-                    self.callback['drop_katochan']()
+                    self.request_parent('NOTIFY', 'drop_katochan')
         elif event.button == 3:                
-            if event.type == gtk.gdk.BUTTON_PRESS:
+            if event.type == Gdk.EventType.BUTTON_PRESS:
                 self.__menu.popup(event.button)
         return True
 
     def set_surface(self):
         if self.id[1] is not None:
-            path = os.path.join(self.dir,
-                                'surface%d%d.png' % (self.id[0], self.id[1]))
+            path = os.path.join(
+                self.dir,
+                os.fsencode('surface{0:d}{1:d}.png'.format(self.id[0], self.id[1])))
             if not os.path.exists(path):
                 self.id[1] = None
                 self.set_surface()
                 return
         else:
-            path = os.path.join(self.dir, 'surface%d.png' % self.id[0])
+            path = os.path.join(self.dir,
+                                os.fsencode('surface{0:d}.png'.format(self.id[0])))
         try:
-            surface_pixbuf = ninix.pix.create_pixbuf_from_file(path)
-            w = max(8, surface_pixbuf.get_width() * self.__scale / 100)
-            h = max(8, surface_pixbuf.get_height() * self.__scale / 100)
-            surface_pixbuf = surface_pixbuf.scale_simple(
-                w, h, gtk.gdk.INTERP_BILINEAR)
+            new_surface = ninix.pix.create_surface_from_file(path)
+            w = max(8, int(new_surface.get_width() * self.__scale / 100))
+            h = max(8, int(new_surface.get_height() * self.__scale / 100))
         except:
-            self.callback['finalize']()
+            self.request_parent('NOTIFY', 'finalize')
             return
         self.w, self.h = w, h
-        mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
-        surface_pixbuf.render_threshold_alpha(
-            mask_pixmap, 0, 0, 0, 0, w, h, 1)
-        if self.window.is_composited():
-            self.window.input_shape_combine_mask(mask_pixmap, 0, 0)
-        else:
-            self.window.shape_combine_mask(mask_pixmap, 0, 0)
         self.darea.set_size_request(self.w, self.h)
-        self.current_mask = mask_pixmap
-        self.current_surface_pixbuf = surface_pixbuf
+        self.image_surface = new_surface
         self.darea.queue_draw()
 
     def set_position(self, reset=0):
@@ -365,13 +376,11 @@ class Skin:
             self.x = left
             self.y = top + scrn_h - self.h
         else:
-            self.x = min(left + scrn_w - self.w, self.x)
-            self.x = max(left, self.x)
-            self.y = min(top + scrn_h - self.h, self.y)
-            self.y = max(top, self.y)
-        self.window.move(self.x, self.y) ## FIXME
+            if not self.omni:
+                self.y = top + scrn_h - self.h
+        self.window.resize_move(self.x, self.y)
 
-    def move(self, x_delta, y_delta): ## FIXME
+    def move(self, x_delta, y_delta):
         self.x = self.x + x_delta
         if self.omni:
             self.y = self.y + y_delta
@@ -381,8 +390,9 @@ class Skin:
         if self.id[1] is not None:
             self.id[1] += 1
         else:
-            if not random.choice(range(100)): ## XXX
-                self.id[1] = 0
+            if random.choice(range(100)): ## XXX
+                return
+            self.id[1] = 0
         self.set_surface()
 
     def start(self): ## FIXME
@@ -394,18 +404,23 @@ class Skin:
         self.set_surface()
 
     def button_release(self,  widget, event):
+        if self.dragged:
+            self.dragged = False
+            self.set_position()
         self.x_root = None
         self.y_root = None
         return True
 
-    def motion_notify(self,  widget, event):
+    def motion_notify(self, widget, event):
         if event.is_hint:
-            x, y, state = self.darea.window.get_pointer()
+            ## FIXME: Use get_device_position() instead.
+            _, x, y, state = widget.get_window().get_pointer()
         else:
-            x, y, state = event.x, event.y, event.state
-        if state & gtk.gdk.BUTTON1_MASK:
+            x, y, state = event.x, event.y, event.get_state()
+        if state & Gdk.ModifierType.BUTTON1_MASK:
             if self.x_root is not None and \
                self.y_root is not None:
+                self.dragged = True
                 x_delta = int(event.x_root - self.x_root)
                 y_delta = int(event.y_root - self.y_root)
                 self.move(x_delta, y_delta)
@@ -441,10 +456,10 @@ class Katochan:
                      'other'      # 上記カテゴリに当てはまらないもの
                      ]
 
-    def __init__(self, callback, target):
+    def __init__(self, target):
         self.side = 0
         self.target = target
-        self.callback = callback
+        self.request_parent = lambda *a: None # dummy
         self.settings = {}
         self.settings['state'] = 'before'
         self.settings['fall.type'] = 'gravity'
@@ -457,6 +472,9 @@ class Katochan:
         self.__scale = 100
         self.loaded = False
 
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
     def get_name(self):
         return self.data['name']
 
@@ -484,16 +502,13 @@ class Katochan:
     def delete(self, widget, event):
         self.destroy()
 
-    def redraw(self, darea, event):
-        cr = darea.window.cairo_create()
-        cr.save()
-        cr.set_operator(cairo.OPERATOR_CLEAR)
+    def redraw(self, widget, cr):
+        scale = self.__scale
+        cr.scale(scale / 100.0, scale / 100.0)
+        cr.set_source_surface(self.image_surface, 0, 0)
+        cr.set_operator(cairo.OPERATOR_SOURCE)
         cr.paint()
-        cr.restore()
-        cr.set_source_pixbuf(self.current_surface_pixbuf, 0, 0)
-        ##cr.paint_with_alpha(self.__alpha_channel)
-        cr.paint()
-        del cr
+        self.window.update_shape(cr.get_target())
 
     def set_movement(self, timing):
         key = ''.join((timing, 'fall.type'))
@@ -533,46 +548,36 @@ class Katochan:
         target_x, target_y = self.target.get_surface_position(self.side)
         target_w, target_h = self.target.get_surface_size(self.side)
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
-        self.x = target_x + target_w / 2 - self.w / 2 + \
-                 self.offset_x * self.__scale / 100
-        self.y = top + self.offset_y * self.__scale / 100
-        self.window.move(self.x, self.y) ## FIXME
+        self.x = target_x + target_w // 2 - self.w // 2 + \
+                 int(self.offset_x * self.__scale / 100)
+        self.y = top + int(self.offset_y * self.__scale / 100)
+        self.window.resize_move(self.x, self.y)
 
     def set_surface(self):
-        path = os.path.join(self.data['dir'], 'surface%d.png' % self.id)
+        path = os.path.join(self.data['dir'],
+                            os.fsencode('surface{0:d}.png'.format(self.id)))
         try:
-            surface_pixbuf = ninix.pix.create_pixbuf_from_file(path)
-            w = max(8, surface_pixbuf.get_width() * self.__scale / 100)
-            h = max(8, surface_pixbuf.get_height() * self.__scale / 100)
-            surface_pixbuf = surface_pixbuf.scale_simple(
-                w, h, gtk.gdk.INTERP_BILINEAR)
+            new_surface = ninix.pix.create_surface_from_file(path)
+            w = max(8, int(new_surface.get_width() * self.__scale / 100))
+            h = max(8, int(new_surface.get_height() * self.__scale / 100))
         except:
-            self.callback['finalize']()
+            self.request_parent('NOTIFY', 'finalize')
             return
         self.w, self.h = w, h
-        mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
-        surface_pixbuf.render_threshold_alpha(
-            mask_pixmap, 0, 0, 0, 0, w, h, 1)
-        if self.window.is_composited():
-            self.window.input_shape_combine_mask(mask_pixmap, 0, 0)
-        else:
-            self.window.shape_combine_mask(mask_pixmap, 0, 0)
         self.darea.set_size_request(self.w, self.h)
         #self.darea.queue_draw_area(0, 0, self.w, self.h)
-        self.current_mask = mask_pixmap
-        self.current_surface_pixbuf = surface_pixbuf
+        self.image_surface = new_surface
         self.darea.queue_draw()
 
     def load(self, data):
         self.data = data
         self.__scale = self.target.get_surface_scale()
         self.set_state('before')
-        ##print 'DATA:', self.data
         if 'category' in self.data:
             category = self.data['category'].split(',')
             if category:
                 if category[0] not in self.CATEGORY_LIST:
-                    print 'WARNING: unknown major category - ', category[0]
+                    logging.warning('WARNING: unknown major category - {0}'.format(category[0]))
                     ##self.data['category'] = self.CATEGORY_LIST[-1]
             else:
                 self.data['category'] = self.CATEGORY_LIST[-1]
@@ -587,8 +592,8 @@ class Katochan:
                 self.side = 0 # XXX
         else:
             self.side = 0 # XXX
-        if self.callback['get_mode']() == 1:
-            self.callback['send_event']('Emerge')
+        if self.request_parent('GET', 'get_mode') == 1:
+            self.request_parent('NOTIFY', 'send_event', 'Emerge')
         else:
             if 'before.script' in self.data:
                 pass ## FIXME
@@ -618,14 +623,14 @@ class Katochan:
         self.window.set_skip_taskbar_hint(True) # XXX
         self.window.set_focus_on_map(False)
         self.window.connect('delete_event', self.delete)
-        self.darea = gtk.DrawingArea()
-        self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
-        self.darea.connect('expose_event', self.redraw)
+        self.darea = Gtk.DrawingArea()
+        self.darea.set_events(Gdk.EventMask.EXPOSURE_MASK)
+        self.darea.connect('draw', self.redraw)
         self.darea.show() ## FIXME
         self.window.add(self.darea)
         self.darea.realize()
         self.window.realize()
-        self.window.window.set_back_pixmap(None, False)
+        self.window.get_window().set_background_pattern(None) # XXX
         self.window.show()
         self.id = 0
         self.set_surface()
@@ -645,7 +650,6 @@ class Katochan:
         pass
 
     def update_position(self): ## FIXME
-        ##print 'MOVE:', self.settings
         if self.settings['slide.type'] == 'leaf':
             pass
         else:
@@ -660,15 +664,14 @@ class Katochan:
                 pass ## FIXME
             else:
                 pass
-        ##print 'POSITION:', self.x, self.y
-        self.window.move(self.x, self.y)
+        self.window.resize_move(self.x, self.y)
 
     def check_collision(self): ## FIXME: check self position
         for side in [0, 1]:
             target_x, target_y = self.target.get_surface_position(side)
             target_w, target_h = self.target.get_surface_size(side)
-            center_x = self.x + self.w / 2
-            center_y = self.y + self.h / 2
+            center_x = self.x + self.w // 2
+            center_y = self.y + self.h // 2
             if target_x < center_x < target_x + target_w and \
                target_y < center_y < target_y + target_h:
                 self.side = side
@@ -678,10 +681,10 @@ class Katochan:
 
     def check_mikire(self):
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
-        if self.x + self.w - self.w / 3 > left + scrn_w or \
-                self.x + self.w / 3 < left or \
-                self.y + self.h - self.h / 3 > top + scrn_h or \
-                self.y + self.h / 3 < top:
+        if self.x + self.w - self.w // 3 > left + scrn_w or \
+                self.x + self.w // 3 < left or \
+                self.y + self.h - self.h // 3 > top + scrn_h or \
+                self.y + self.h // 3 < top:
             return 1
         else:
             return 0
@@ -693,10 +696,10 @@ class Katochan:
             if self.check_collision():
                 self.set_state('hit')
                 self.hit = 1
-                if self.callback['get_mode']() == 1:
+                if self.request_parent('GET', 'get_mode') == 1:
                     self.id = 1
                     self.set_surface()
-                    self.callback['send_event']('Hit')
+                    self.request_parent('NOTIFY', 'send_event', 'Hit')
                 else:
                     pass ## FIXME
             if self.check_mikire():
@@ -705,10 +708,10 @@ class Katochan:
             if self.hit_stop >= self.data.get('hit.waittime', 0):
                 self.set_state('after')
                 self.set_movement('after')
-                if self.callback['get_mode'] == 1:
+                if self.request_parent('GET', 'get_mode') == 1:
                     self.id = 2
                     self.set_surface()
-                    self.callback['send_event']('Drop')
+                    self.request_parent('NOTIFY', 'send_event', 'Drop')
                 else:
                     pass ## FIXME
             else:
@@ -720,18 +723,18 @@ class Katochan:
             if self.check_mikire():
                 self.set_state('end')
         elif self.settings['state'] == 'end':
-            if self.callback['get_mode']() == 1:
-                self.callback['send_event']('Vanish')
+            if self.request_parent('GET', 'get_mode') == 1:
+                self.request_parent('NOTIFY', 'send_event', 'Vanish')
             else:
                 pass ## FIXME
-            self.callback['delete_katochan']()
+            self.request_parent('NOTIFY', 'delete_katochan')
             return False
         elif self.settings['state'] == 'dodge':
-            if self.callback['get_mode'] == 1:
-                self.callback['send_event']('Dodge')
+            if self.request_parent('GET', 'get_mode') == 1:
+                self.request_parent('NOTIFY', 'send_event', 'Dodge')
             else:
                 pass ## FIXME
-            self.callback['delete_katochan']()
+            self.request_parent('NOTIFY', 'delete_katochan')
             return False
         else:
             pass ## check collision and mikire
