@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -17,7 +17,7 @@ import re
 import sys
 import random
 
-import glib
+from gi.repository import GLib
 
 
 class Controler:
@@ -26,6 +26,7 @@ class Controler:
 
     def __init__(self, seriko):
         self.seriko = seriko
+        self.request_parent = lambda *a: None # dummy
         self.exclusive_actor = None
         self.base_id = None
         self.timeout_id = None
@@ -37,10 +38,9 @@ class Controler:
         self.active = []
         self.__move = None
         self.__dirty = 1
-        self.__inactive = False
 
-    def is_active(self):
-        return not bool(self.__inactive)
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
 
     def set_base_id(self, window, surface_id):
         if surface_id == '-2':
@@ -58,21 +58,6 @@ class Controler:
     def append_actor(self, frame, actor):
         self.active.append((frame, actor))
 
-    def set_animation_quality(self, quality, window):
-        if self.timeout_id is not None:
-            glib.source_remove(self.timeout_id)
-        if quality is None:
-            quality = 1.0
-        self.fps = self.DEFAULT_FPS * quality
-        if self.timeout_id is not None:
-            self.timeout_id = glib.timeout_add(
-                int(1000.0 / self.fps), # [msec]
-                self.update, window)
-
-    def set_inactive(self, flag, window):
-        self.__inactive = bool(flag)
-        self.__dirty = 1 # XXX
-
     def update_frame(self, window):
         frame, actor = self.get_actor_next(window)
         last_actor = actor
@@ -86,18 +71,20 @@ class Controler:
 
     def get_actor_next(self, window):
         if self.active:
-            self.active.sort()
+            self.active.sort(key=lambda x: x[0])
             if self.active[0][0] <= self.next_tick:
                 return self.active.pop(0)
         return None, None
 
     def update(self, window):
-        ## FIXME: Use g_source_get_time. (glib.glib_version > (2, 27, 3))
-        current_tick = glib.get_current_time() # [sec]
+        current_tick = GLib.get_monotonic_time() # [microsec]
+        quality = self.request_parent(
+            'GET', 'get_preference', 'animation_quality')
+        self.fps = self.DEFAULT_FPS * quality
         if self.prev_tick == 0: ## First time
             delta_tick = 1000.0 / self.fps # [msec]
         else:
-            delta_tick = (current_tick - self.prev_tick) * 1000.0 # [msec]
+            delta_tick = (current_tick - self.prev_tick) / 1000 # [msec]
         self.next_tick += delta_tick
         self.prev_tick = current_tick
         self.update_frame(window)
@@ -107,7 +94,9 @@ class Controler:
         if self.__move is not None:
             window.move_surface(*self.__move)
             self.__move = None
-        return True
+        self.timeout_id = GLib.timeout_add(int(1000.0 / self.fps), # [msec]
+                                           self.update, window)
+        return False
 
     def lock_exclusive(self, window, actor):
         assert self.exclusive_actor is None
@@ -131,13 +120,13 @@ class Controler:
             pass
         self.__dirty = 1
 
-    def add_overlay(self, window, actor, pixbuf_id, x, y):
-        if pixbuf_id == '-2':
+    def add_overlay(self, window, actor, surface_id, x, y):
+        if surface_id == '-2':
             self.terminate(window)
-        if pixbuf_id in ['-1', '-2']:
+        if surface_id in ['-1', '-2']:
             self.remove_overlay(actor)
             return
-        self.overlays[actor] = (pixbuf_id, x, y)
+        self.overlays[actor] = (surface_id, x, y)
         self.__dirty = 1
 
     def invoke_actor(self, window, actor):
@@ -228,8 +217,8 @@ class Controler:
         self.invoke_runonce(window)
         self.invoke_always(window)
         if self.timeout_id is not None:
-            glib.source_remove(self.timeout_id)
-        self.timeout_id = glib.timeout_add(
+            GLib.source_remove(self.timeout_id)
+        self.timeout_id = GLib.timeout_add(
             int(1000.0 / self.fps), # [msec]
             self.update, window)
 
@@ -244,18 +233,20 @@ class Controler:
 
     def destroy(self):
         if self.timeout_id is not None:
-            glib.source_remove(self.timeout_id)
+            GLib.source_remove(self.timeout_id)
             self.timeout_id = None
 
     def iter_overlays(self):
-        actors = self.overlays.keys()
+        actors = list(self.overlays.keys())
         actors[:] = [(actor.get_id(), actor) for actor in actors]
         actors.sort()
         actors[:] = [actor for actor_id, actor in actors]
         for actor in actors:
-            pixbuf_id, x, y = self.overlays[actor]
-            ##print 'actor=%d, id=%s, x=%d, y=%d' % (actor.get_id(), pixbuf_id, x, y)
-            yield pixbuf_id, x, y
+            surface_id, x, y = self.overlays[actor]
+            ##logging.debug(
+            ##    'actor={0:d}, id={1}, x={2:d}, y={3:d}'.format(
+            ##        actor.get_id(), surface_id, x, y))
+            yield surface_id, x, y
 
 
 class Actor:
@@ -324,7 +315,7 @@ class Actor:
         elif method == 'alternativestart':
             window.invoke(random.choice(args), update=1)
         else:
-            raise RuntimeError, 'should not reach here'
+            raise RuntimeError('should not reach here')
         self.last_method = method
 
 
@@ -472,7 +463,7 @@ re_seriko2_pattern = re.compile(r'^(overlay|overlayfast|base|move|start|alternat
 def get_actors(config):
     version = None
     buf = []
-    for key, value in config.iteritems():
+    for key, value in config.items():
         if version == 1:
             match = re_seriko_interval.match(key)
         elif version == 2:
@@ -535,7 +526,7 @@ def get_actors(config):
                 else:
                     match = re_seriko2_pattern.match(pattern)
                 if not match:
-                    raise ValueError, 'unsupported pattern: %s' % pattern
+                    raise ValueError('unsupported pattern: {0}'.format(pattern))
                 if version == 1:
                     surface = str(int(match.group(1)))
                     interval = abs(int(match.group(2))) * 10
@@ -549,12 +540,12 @@ def get_actors(config):
                 if method == 'start':
                     group = match.group(4)
                     if group is None:
-                        raise ValueError, 'syntax error: %s' % pattern
+                        raise ValueError('syntax error: {0}'.format(pattern))
                     args = [int(group)]
                 elif method == 'alternativestart':
                     args = match.group(6)
                     if args is None:
-                        raise ValueError, 'syntax error: %s' % pattern
+                        raise ValueError('syntax error: {0}'.format(pattern))
                     args = [int(s) for s in args[1:-1].split('.')]
                 else:
                     if surface in ['-1', '-2']:
@@ -565,12 +556,12 @@ def get_actors(config):
                         y = int(match.group(5) or 0)
                     args = [x, y]
                 actor.add_pattern(surface, interval, method, args)
-        except ValueError, error:
+        except ValueError as error:
             logging.error(''.join(('seriko.py: ', str(error))))
             continue
         if not actor.get_patterns():
             logging.error(
-                'seriko.py: animation group #%d has no pattern (ignored)' % actor_id)
+                'seriko.py: animation group #{0:d} has no pattern (ignored)'.format(actor_id))
             continue
         actors.append(actor)
     actors[:] = [(actor.get_id(), actor) for actor in actors]
@@ -589,7 +580,7 @@ re_mayuna2_pattern = re.compile(r'^(bind|add|reduce|insert)\s*,\s*([0-9]+|-[12])
 def get_mayuna(config):
     version = None
     buf = []
-    for key, value in config.iteritems():
+    for key, value in config.items():
         if version == 1:
             match = re_mayuna_interval.match(key)
         elif version == 2:
@@ -632,7 +623,7 @@ def get_mayuna(config):
                 else:
                     match = re_mayuna2_pattern.match(pattern)
                 if not match:
-                    raise ValueError, 'unsupported pattern: %s' % pattern
+                    raise ValueError('unsupported pattern: {0}'.format(pattern))
                 if version == 1:
                     surface = str(int(match.group(1)))
                     interval = abs(int(match.group(2))) * 10
@@ -652,12 +643,12 @@ def get_mayuna(config):
                         y = int(match.group(5) or 0)
                     args = [x, y]
                 actor.add_pattern(surface, interval, method, args)
-        except ValueError, error:
+        except ValueError as error:
             logging.error(''.join(('seriko.py: ', str(error))))
             continue
         if not actor.get_patterns():
             logging.error(
-                'seriko.py: animation group #%d has no pattern (ignored)' % mayuna_id)
+                'seriko.py: animation group #{0:d} has no pattern (ignored)'.format(mayuna_id))
             continue
         mayuna.append(actor)
     mayuna[:] = [(actor.get_id(), actor) for actor in mayuna]
@@ -667,25 +658,25 @@ def get_mayuna(config):
 
 # find ~/.ninix -name 'surface*a.txt' | xargs python seriko.py
 def test():
-    import ninix.config
+    import ninix.config as config
     if len(sys.argv) == 1:
-        print 'Usage:', sys.argv[0], '[surface??a.txt ...]'
+        print('Usage:', sys.argv[0], '[surface??a.txt ...]')
     for filename in sys.argv[1:]:
-        print 'Reading', filename, '...'
-        for actor in get_actors(ninix.config.create_from_file(filename)):
-            print '#%d' % actor.get_id(),
-            print actor.__class__.__name__,
-            print '(%s)' % actor.get_interval()
-            print 'number of patterns =', len(actor.get_patterns())
+        print('Reading', filename, '...')
+        for actor in get_actors(config.create_from_file(filename)):
+            print('#{0:d}'.format(actor.get_id()),)
+            print(actor.__class__.__name__,)
+            print('({0})'.format(actor.get_interval()))
+            print('number of patterns =', len(actor.get_patterns()))
             for pattern in actor.get_patterns():
-                print 'surface=%s, interval=%d, method=%s, args=%s' % pattern
-        for actor in get_mayuna(ninix.config.create_from_file(filename)):
-            print '#%d' % actor.get_id(),
-            print actor.__class__.__name__,
-            print '(%s)' % actor.get_interval()
-            print 'number of patterns =', len(actor.get_patterns())
+                print('surface={0}, interval={1:d}, method={2}, args={3}'.format(*pattern))
+        for actor in get_mayuna(config.create_from_file(filename)):
+            print('#{0:d}'.format(actor.get_id()),)
+            print(actor.__class__.__name__,)
+            print('({0})'.format(actor.get_interval()))
+            print('number of patterns =', len(actor.get_patterns()))
             for pattern in actor.get_patterns():
-                print 'surface=%s, interval=%d, method=%s, args=%s' % pattern
+                print('surface={0}, interval={1:d}, method={2}, args={3}'.format(*pattern))
 
 
 if __name__ == '__main__':
